package me.captain.dnc;

import java.util.ArrayList;

import org.bukkit.ChatColor;
import org.bukkit.Material;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.getspout.spoutapi.player.SpoutPlayer;

/**
 * Command executor for the DispNameChanger Plugin.
 * 
 * Valid commands:aliases that this executor handles are
 * <code>'rename':'newname'</code>, <code>'reset':'resetname':</code>, and
 * <code>'check':'checkname':'namecheck'</code>.
 * 
 * @author captainawesome7, itsatacoshop247, Daxiongmao87, Luke Zwekii,
 *         Sammy, SniperFodder
 * 
 */
public class DispNameCE implements CommandExecutor
{
	private DispNameChanger plugin;
	
	/**
	 * Constructs a new DispNameCE.
	 * 
	 * @param plugin
	 *            The plugin using this Command Executor.
	 */
	public DispNameCE(DispNameChanger plugin)
	{
		super();
		
		this.plugin = plugin;
	}
	
	/**
	 * Checks to see if the Player can use the command.
	 * 
	 * @param p
	 *            The player to check the permission for.
	 * 
	 * @return true if the player can use the command, false otherwise.
	 */
	public boolean canUseCheckName(Player p)
	{
		if (p == null)
		{
			return true;
		}
		
		return p.hasPermission("dispname.check");
	}
	
	/**
	 * Checks to see if the Player can use the command.
	 * 
	 * @param p
	 *            The player to check the permission for.
	 * 
	 * @return true if the player can use the command, false otherwise.
	 */
	public boolean canUseChangeName(Player p)
	{
		if (p == null)
		{
			return true;
		}
		
		return p.hasPermission("dispname.change");
	}
	
	/**
	 * Checks to see if the Player can use the command.
	 * 
	 * @param p
	 *            The player to check the permission for.
	 * 
	 * @return true if the player can use the command, false otherwise.
	 */
	public boolean canUseChangeNameSpace(Player p)
	{
		if (p == null)
		{
			return true;
		}
		
		return p.hasPermission("dispname.changespace");
	}
	
	/**
	 * Checks to see if the Player can use the command.
	 * 
	 * @param p
	 *            The player to check the permission for.
	 * 
	 * @return true if the player can use the command, false otherwise.
	 */
	public boolean canUseChangeNameOther(Player p)
	{
		if (p == null)
		{
			return true;
		}
		
		return p.hasPermission("dispname.changeother");
	}
	
	@Override
	public boolean onCommand(CommandSender sender, Command cmd,
			String commandLabel, String[] args)
	{
		
		Player changer = null;
		
		if (sender instanceof Player)
		{
			changer = (Player) sender;
		}
		
		// Check for the rename command.
		if (commandLabel.equalsIgnoreCase("rename")
				|| commandLabel.equalsIgnoreCase("newname"))
		{
			// parse Arguments into <target> <Name>
			String[] saArgs = parseArguments(args);
			
			// Ensure that the user can run the command.
			if (canUseChangeName(changer))
			{
				// Changing own name.
				if (args.length == 1)
				{
					// Ensure we aren't console.
					if (changer == null)
					{
						sender.sendMessage("Can't change console's name.");
						
						return false;
					}
				}
				
				if (saArgs == null)
				{
					return false;
				}
				
				switch (saArgs.length)
				{
				
				/*
				 * If Args returned is of size 1, changing own name.
				 */
				case 1:
					changeDisplayName(changer, saArgs[0]);
					
					return true;
					
					/*
					 * If Args returned is of size 2, Changing someone
					 * else's name.
					 */
				case 2:
					// Ensure we can change other names.
					if (canUseChangeNameOther(changer))
					{
						// Pull the target we are changing.
						Player[] players = checkName(saArgs[0]);
						
						if (players == null)
						{
							if (changer != null)
							{
								changer.sendMessage(ChatColor.RED
										+ "User does not exist, or is not online!");
							}
							else
							{
								sender.sendMessage("User does not exist, or is not online!");
							}
							
							return true;
						}
						
						// Ensure there is only one target.
						if (players.length == 1)
						{
							if (saArgs[1].contains(" "))
							{
								// Ensure we can change spaces.
								if (canUseChangeNameSpace(changer))
								{
									// Change the targets name.
									changeDisplayName(changer, players[0],
											saArgs[0], saArgs[1]);
									
									return true;
								}
								else
								{
									changer.sendMessage(ChatColor.RED
											+ "You do not have permission to change "
											+ "names with spaces in them!");
									
									return true;
								}
							}
							else
							{
								if (changer != null
										&& changer.equals(players[0]))
								{
									changeDisplayName(changer, saArgs[1]);
								}
								else
								{
									// Change the targets name.
									changeDisplayName(changer, players[0],
											saArgs[0], saArgs[1]);
								}
								
								return true;
							}
							
						}
						else
						{
							if (changer != null)
							{
								changer.sendMessage(ChatColor.RED
										+ "Too many users match target!");
							}
							else
							{
								sender.sendMessage("Too many users match target!");
							}
							return true;
						}
					}
					else
					{
						changer.sendMessage(ChatColor.RED
								+ "You do not have permission to change other player names!");
						
						return true;
					}
				}
			}
			else
			{
				changer.sendMessage(ChatColor.RED
						+ "You don't have permisson to change names!");
				
				return true;
			}
		}
		// Reset name command.
		else if (commandLabel.equalsIgnoreCase("resetname")
				|| commandLabel.equalsIgnoreCase("reset"))
		{
			if (args.length == 0)
			{
				if (changer == null)
				{
					sender.sendMessage("Can not rename the console!");
					
					return false;
				}
				else
				{
					if (canUseChangeName(changer))
					{
						changeDisplayName(changer, changer.getName());
						
						return true;
					}
					else
					{
						changer.sendMessage(ChatColor.RED
								+ "You don't have permisson to change names!");
						
						return true;
					}
				}
			}
			else
			{
				if (canUseChangeName(changer)
						&& canUseChangeNameOther(changer))
				{
					String[] saArgs = parseArguments(args);
					
					if (saArgs == null)
					{
						if (changer != null)
						{
							changer.sendMessage(ChatColor.RED
									+ "User does not exist, or is not online!");
						}
						else
						{
							sender.sendMessage("User does not exist, or is not online!");
						}
						
						return false;
					}
					if (saArgs.length > 1)
					{
						if (changer != null)
						{
							changer.sendMessage(ChatColor.RED
									+ "To many arguments!");
						}
						else
						{
							sender.sendMessage("To many arguments!");
						}
						
						return false;
					}
					else if (saArgs.length == 1)
					{
						Player[] players = checkName(saArgs[0]);
						
						if (players.length > 1)
						{
							if (changer != null)
							{
								changer.sendMessage(ChatColor.RED
										+ "Too many users match target!");
							}
							else
							{
								sender.sendMessage("Too many users match target!");
							}
							
							return true;
						}
						else
						{
							changeDisplayName(changer, players[0], saArgs[0],
									players[0].getName());
							
							return true;
						}
					}
				}
				else
				{
					changer.sendMessage(ChatColor.RED
							+ "You do not have permission to change other player names!");
					
					return true;
				}
			}
		}
		// Check name command.
		else if (commandLabel.equalsIgnoreCase("checkname")
				|| commandLabel.equalsIgnoreCase("check")
				|| commandLabel.equalsIgnoreCase("namecheck"))
		{
			if (canUseCheckName(changer))
			{
				String[] saArgs = parseArguments(args);
				
				if (saArgs == null)
				{
					return false;
				}
				if (saArgs.length == 1)
				{
					Player[] players = checkName(saArgs[0]);
					
					if (players == null)
					{
						sender.sendMessage(ChatColor.RED
								+ "Unable to find specified user.");
						
						return true;
					}
					else if (players.length == 1)
					{
						String sName = players[0].getName();
						
						String sDisplay = players[0].getDisplayName();
						
						StringBuilder sb = new StringBuilder();
						
						if (changer != null)
						{
							sb.append(ChatColor.GREEN);
						}
						
						if (sDisplay.equals(saArgs[0]))
						{
							sb.append(sDisplay);
							
							sb.append(" is really: ");
							
							sb.append(sName);
							
							sb.append(".");
						}
						else
						{
							sb.append(sName);
							
							sb.append(" is really: ");
							
							sb.append(sDisplay);
							
							sb.append(".");
						}
						
						if (changer != null)
						{
							changer.sendMessage(sb.toString());
						}
						else
						{
							sender.sendMessage(sb.toString());
						}
						
						return true;
					}
					else
					{
						if (changer != null)
						{
							changer.sendMessage(ChatColor.RED
									+ "More than 1 user found.");
						}
						else
						{
							sender.sendMessage("More than 1 user found.");
						}
						
						StringBuilder sb = new StringBuilder();
						
						if (changer != null)
						{
							sb.append(ChatColor.GREEN);
						}
						
						sb.append(saArgs[0]);
						
						sb.append(" could be ");
						
						for (Player p : players)
						{
							sb.append(p.getName());
							
							sb.append(", ");
						}
						
						String s = sb.toString();
						
						s = s.substring(0, (s.length() - 2));
						
						s += ".";
						
						if (changer != null)
						{
							changer.sendMessage(s);
						}
						else
						{
							sender.sendMessage(s);
						}
					}
				}
				else
				{
					if (changer != null)
					{
						changer.sendMessage(ChatColor.RED
								+ "To many arguments!");
					}
					else
					{
						sender.sendMessage("To many arguments!");
					}
					
					return false;
				}
				
			}
			else
			{
				changer.sendMessage(ChatColor.RED
						+ "You don't have permisson to check names!");
			}
			
			return true;
		}
		
		return false;
	}
	
	/*
	 * Changes the display name of the target.
	 * 
	 * @param target the player to change.
	 * 
	 * @param newName the new name to use.
	 */
	private void changeDisplayName(Player target, String newName)
	{
		changeDisplayName(null, target, null, newName);
	}
	
	/*
	 * Changes the display name of the target and sends notifications.
	 * 
	 * @param caller The person trying to make the change.
	 * 
	 * @param target The intended target of the change.
	 * 
	 * @param oldName The old name of the target.
	 * 
	 * @param newName The new name to use.
	 */
	private void changeDisplayName(Player caller, Player target,
			String oldName, String newName)
	{
		String spoutName = newName.replaceAll("(&([a-f0-9]))", "$2");
		
		target.setDisplayName(spoutName);
		
		StringBuilder sbTarget = new StringBuilder();
		
		StringBuilder sbCaller;
		
		sbTarget.append(ChatColor.GREEN);
		
		sbTarget.append("Your name is now ");
		
		sbTarget.append(newName);
		
		sbTarget.append(".");
		
		target.sendMessage(sbTarget.toString());
		
		if (caller != null && !caller.equals(target))
		{
			sbCaller = new StringBuilder();
			
			sbCaller.append(ChatColor.GREEN);
			
			sbCaller.append(oldName);
			
			sbCaller.append("'s name is now ");
			
			sbCaller.append(newName);
			
			sbCaller.append(".");
			
			caller.sendMessage(sbCaller.toString());
		}
		
		if (plugin.isSpoutEnabled() == true)
		{
			SpoutPlayer spoutTarget = (SpoutPlayer) target;
			
			if (spoutTarget.isSpoutCraftEnabled())
			{
				spoutTarget.sendNotification("Your name is:\t", spoutName,
						Material.DIAMOND);
			}
			
			if (caller != null && !caller.equals(target))
			{
				SpoutPlayer spoutCaller = (SpoutPlayer) caller;
				
				if (spoutCaller.isSpoutCraftEnabled())
				{
					StringBuilder sbSpout = new StringBuilder();
					
					sbSpout.append(oldName);
					
					sbSpout.append("'s name:\t");
					
					spoutCaller.sendNotification(sbSpout.toString(),
							spoutName, Material.DIAMOND);
				}
			}
			
			plugin.getApperanceManager()
					.setGlobalTitle(spoutTarget, spoutName);
		}
		
		DP pClass = (DP) plugin.getDatabase().find(DP.class).where()
				.ieq("PlayerName", target.getName()).findUnique();
		
		if (pClass == null)
		{
			pClass = new DP();
		}
		
		pClass.setPlayerName(target.getName());
		
		pClass.setDisplayName(newName);
		
		plugin.getDatabase().save(pClass);
	}
	
	/*
	 * Checks for a valid player based upon a given name.
	 * 
	 * @param name the name to check for.
	 * 
	 * @return an array of players matching the name.
	 */
	private Player[] checkName(String name)
	{
		if (name == null)
		{
			throw new IllegalArgumentException("Name can not be null.");
		}
		
		Player[] players = plugin.getServer().getOnlinePlayers();
		
		ArrayList<Player> alPlayers = new ArrayList<Player>();
		
		String sName;
		
		String sDisplayName;
		
		for (Player p : players)
		{
			sName = p.getName().toLowerCase();
			
			sDisplayName = p.getDisplayName().toLowerCase();
			
			if (sDisplayName.equals(name.toLowerCase()))
			{
				alPlayers.add(p);
			}
			else if (sName.equals(name.toLowerCase()))
			{
				alPlayers.add(p);
			}
		}
		
		alPlayers.trimToSize();
		
		if (alPlayers.size() == 0)
		{
			return null;
		}
		
		Player[] p = new Player[alPlayers.size()];
		
		alPlayers.toArray(p);
		
		return p;
	}
	
	/*
	 * Parses the argument string, returning 1 or 2 arguments matching
	 * <target> <name>
	 * 
	 * @param args the arguments string from the command.
	 * 
	 * @return A String array containing the arguments <target> and/or
	 * <name> in that order.
	 */
	private String[] parseArguments(String[] args)
	{
		String sTarget, sName = "";
		
		if (args.length == 0)
		{
			return null;
		}
		// Only one argument.
		else if (args.length == 1)
		{
			sName = args[0].replace("\"", "");
			
			String[] sa =
			{ sName };
			
			return sa;
		}
		// Check to see if target/name given.
		else if (args.length == 2)
		{
			if (args[0].startsWith("\""))
			{
				if (!args[0].endsWith("\""))
				{
					sTarget = args[0] + " " + args[1];
					
					sTarget = sTarget.replace("\"", "");
					
					String[] sa =
					{ sTarget };
					
					return sa;
				}
				else
				{
					sTarget = args[0].replace("\"", "");
					
					sName = args[1].replace("\"", "");
					
					String[] sa =
					{ sTarget, sName };
					
					return sa;
				}
			}
			// <target> + <name> given.
			else
			{
				String[] sa =
				{ args[0], args[1] };
				
				return sa;
			}
		}
		// Arguments with spaces given.
		else
		{
			// Check to see if first argument has space.
			if (args[0].startsWith("\""))
			{
				sTarget = "";
				
				int iLoop1;
				
				// Look for the end quote for the first argument.
				for (iLoop1 = 0; iLoop1 < args.length; iLoop1++)
				{
					sTarget += args[iLoop1];
					
					if (args[iLoop1].endsWith("\""))
					{
						break;
					}
					
					sTarget += " ";
				}
				
				sTarget = sTarget.replace("\"", "");
				
				iLoop1++;
				
				// Check to see if really only one argument.
				if (iLoop1 >= args.length)
				{
					
					String[] sa =
					{ sTarget };
					
					return sa;
				}
				
				// Parse out second argument.
				for (int iLoop2 = iLoop1; iLoop2 < args.length; iLoop2++)
				{
					
					sName += args[iLoop2] + " ";
				}
				
				int iIndex = sName.lastIndexOf(" ");
				
				if (iIndex > -1)
				{
					sName = sName.substring(0, iIndex);
				}
				
				sName = sName.replace("\"", "");
				
				String[] sa =
				{ sTarget, sName };
				
				return sa;
			}
			// First arg doesn't have space.
			else
			{
				sTarget = args[0].replace("\"", "");
				
				if (args[1].startsWith("\""))
				{
					int iLoop1 = 0;
					
					for (iLoop1 = 1; iLoop1 < args.length; iLoop1++)
					{
						sName += args[iLoop1];
						
						if (args[iLoop1].endsWith("\""))
						{
							break;
						}
						
						sName += " ";
					}
					
					if (iLoop1 < (args.length - 1))
					{
						return null;
					}
					
					sName = sName.replace("\"", "");
					
					String[] sa =
					{ sTarget, sName };
					
					return sa;
				}
				else
				{
					return null;
				}
			}
		}
	}
	
}
