package me.captain.dnc;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import javax.persistence.PersistenceException;
import org.bukkit.event.Event;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;
import org.getspout.spoutapi.SpoutManager;
import org.getspout.spoutapi.player.AppearanceManager;

/**
 * Allows a user to change their display name, or the display name of
 * others. Requires spout integration for changing the name above the head.
 * 
 * @author captainawesome7, itsatacoshop247, Daxiongmao87, Luke Zwekii,
 *         Sammy, SniperFodder
 * 
 */
public class DispNameChanger extends JavaPlugin
{
	private AppearanceManager am;
	
	private SpoutManager spout;
	
	private final DispNameCE executor;
	
	private final DPL playerlistener;
	
	private static DispNameChanger instance;
	
	private static final Logger log = Logger.getLogger("Minecraft");
	
	private boolean useSpout;
	
	/**
	 * Constructs a new DispNameChanger.
	 */
	public DispNameChanger()
	{
		super();
		
		executor = new DispNameCE(this);
		
		playerlistener = new DPL(this);
		
		instance = this;
	}
	
	/**
	 * Gets the current AppearanceManager.
	 * 
	 * @return an AppearanceManager.
	 */
	public AppearanceManager getApperanceManager()
	{
		return am;
	}

	@SuppressWarnings(
	{ "unchecked", "rawtypes" })
	@Override
	public List<Class<?>> getDatabaseClasses()
	{
		List list = new ArrayList();
		
		list.add(DP.class);
		
		return list;
	}
	
	/**
	 * Gets the current SpoutManager.
	 * 
	 * @return a SpoutManager.
	 */
	public SpoutManager getSpoutManager()
	{
		return spout;
	}
	
	/**
	 * Returns whether or not spout is on the server.
	 * 
	 * @return true if spout is loaded, false otherwise.
	 */
	public boolean isSpoutEnabled()
	{
		return useSpout;
	}
	
	@Override
	public void onDisable()
	{
		log.info(new StringBuilder()
				.append("[DispNameChanger] DispNameChanger version ")
				.append(getDescription().getVersion())
				.append(" has been disabled!").toString());
	}
	
	@Override
	public void onEnable()
	{
		PluginManager pm = getServer().getPluginManager();
		
		setupDatabase();
		
		pm.registerEvent(Event.Type.PLAYER_JOIN, this.playerlistener,
				Event.Priority.Normal, this);
		
		pm.registerEvent(Event.Type.PLAYER_QUIT, this.playerlistener,
				Event.Priority.Normal, this);
		
		getCommand("rename").setExecutor(executor);
		
		getCommand("reset").setExecutor(executor);
		
		getCommand("check").setExecutor(executor);
		
		Plugin Spout = pm.getPlugin("Spout");
		
		if (Spout == null)
		{
			this.useSpout = false;
			
			log.info("[DispNameChanger] Spout not detected! Not using spout features.");
		}
		else
		{
			this.useSpout = true;
			
			log.info(new StringBuilder()
					.append("[DispNameChanger] Spout detected! Using version ")
					.append(Spout.getDescription().getVersion()).append("!")
					.toString());
			
			this.am = SpoutManager.getAppearanceManager();
		}
		
		log.info(new StringBuilder().append("[DispNameChanger] Version ")
				.append(getDescription().getVersion()).append(" by ")
				.append(getDescription().getAuthors().toString())
				.append(" has been enabled!").toString());
	}
	
	/**
	 * Returns the current plugin instance.
	 * 
	 * @return the DispNameChanger plugin.
	 */
	public static DispNameChanger getInstance()
	{
		return instance;
	}

	/*
	 * Tries to load the database, and if unable to find it, creates a new
	 * one.
	 */
	private void setupDatabase()
	{
		try
		{
			getDatabase().find(DP.class).findRowCount();
		}
		catch (PersistenceException ex)
		{
			System.out.println(new StringBuilder()
					.append("Installing database for ")
					.append(getDescription().getName())
					.append(" due to first time usage").toString());
			
			installDDL();
		}
	}
}